@@ -1,13 +1,15 @@
 //! Actor errors.
 
-use super::schemas::internal_protocol::_Any;
-
-#[doc(inline)]
-pub use super::schemas::internal_protocol::Error;
+use super::actor::AnyValue;
+use super::language;
+use super::schemas::protocol as P;
 
 use preserves::value::NestedValue;
 use preserves::value::Value;
-use preserves_schema::support::ParseError;
+use preserves_schema::Codec;
+use preserves_schema::ParseError;
+
+pub type Error = P::Error<AnyValue>;
 
 impl std::error::Error for Error {}
 
@@ -20,10 +22,10 @@ impl std::fmt::Display for Error {
 /// Construct an [`Error`] with the given `message` and `detail`.
 ///
 /// When no relevant detail exists, convention is to set `detail` to `false`.
-pub fn error<Detail>(message: &str, detail: Detail) -> Error where _Any: From<Detail> {
+pub fn error<Detail>(message: &str, detail: Detail) -> Error where AnyValue: From<Detail> {
     Error {
         message: message.to_owned(),
-        detail: _Any::from(detail),
+        detail: AnyValue::from(detail),
     }
 }
 
@@ -33,16 +35,16 @@ pub fn error<Detail>(message: &str, detail: Detail) -> Error where _Any: From<De
 /// Used primarily when attempting to perform an
 /// [`Activation`][crate::actor::Activation] on an already-terminated
 /// actor.
-pub fn encode_error(result: Result<(), Error>) -> _Any {
+pub fn encode_error(result: Result<(), Error>) -> AnyValue {
     match result {
         Ok(()) => {
-            let mut r = Value::record(_Any::symbol("Ok"), 1);
-            r.fields_vec_mut().push(Value::record(_Any::symbol("tuple"), 0).finish().wrap());
+            let mut r = Value::record(AnyValue::symbol("Ok"), 1);
+            r.fields_vec_mut().push(Value::record(AnyValue::symbol("tuple"), 0).finish().wrap());
             r.finish().wrap()
         }
         Err(e) => {
-            let mut r = Value::record(_Any::symbol("Err"), 1);
-            r.fields_vec_mut().push((&e).into());
+            let mut r = Value::record(AnyValue::symbol("Err"), 1);
+            r.fields_vec_mut().push(language().unparse(&e));
             r.finish().wrap()
         }
     }
@@ -50,24 +52,24 @@ pub fn encode_error(result: Result<(), Error>) -> _Any {
 
 impl<'a> From<&'a str> for Error {
     fn from(v: &'a str) -> Self {
-        error(v, _Any::new(false))
+        error(v, AnyValue::new(false))
     }
 }
 
 impl From<std::io::Error> for Error {
     fn from(v: std::io::Error) -> Self {
-        error(&format!("{}", v), _Any::new(false))
+        error(&format!("{}", v), AnyValue::new(false))
     }
 }
 
 impl From<ParseError> for Error {
     fn from(v: ParseError) -> Self {
-        error(&format!("{}", v), _Any::new(false))
+        error(&format!("{}", v), AnyValue::new(false))
     }
 }
 
 impl From<preserves::error::Error> for Error {
     fn from(v: preserves::error::Error) -> Self {
-        error(&format!("{}", v), _Any::new(false))
+        error(&format!("{}", v), AnyValue::new(false))
     }
 }
